==> shinqlx-0.6.0.tar.gz: shinqlx-0.6.0/src/ffi/python/embed/set_powerups.rs <==
use super::validate_client_id;
use crate::ffi::c::prelude::*;
use crate::ffi::python::prelude::*;

/// Sets a player's powerups.
#[pyfunction]
#[pyo3(name = "set_powerups")]
pub(crate) fn pyshinqlx_set_powerups(
    py: Python<'_>,
    client_id: i32,
    powerups: &Powerups,
) -> PyResult<bool> {
    py.allow_threads(|| {
        validate_client_id(client_id)?;

        #[cfg_attr(test, allow(clippy::unnecessary_fallible_conversions))]
        let mut opt_game_client = GameEntity::try_from(client_id)
            .ok()
            .and_then(|game_entity| game_entity.get_game_client().ok());
        opt_game_client
            .iter_mut()
            .for_each(|game_client| game_client.set_powerups((*powerups).into()));
        Ok(opt_game_client.is_some())
    })
}

#[cfg(test)]
mod set_powerups_tests {
    use crate::ffi::c::prelude::*;
    use crate::ffi::python::prelude::*;
    use crate::prelude::*;

    use mockall::predicate;
    use pretty_assertions::assert_eq;
    use pyo3::exceptions::{PyEnvironmentError, PyValueError};
    use rstest::rstest;

    #[rstest]
    #[cfg_attr(miri, ignore)]
    #[serial]
    fn set_powerups_when_main_engine_not_initialized(_pyshinqlx_setup: ()) {
        let powerups = Powerups(0, 0, 0, 0, 0, 0);

        Python::with_gil(|py| {
            let result = pyshinqlx_set_powerups(py, 21, &powerups);
            assert!(result.is_err_and(|err| err.is_instance_of::<PyEnvironmentError>(py)));
        });
    }

    #[rstest]
    #[cfg_attr(miri, ignore)]
    #[serial]
    fn set_powerups_for_client_id_too_small(_pyshinqlx_setup: ()) {
        let powerups = Powerups(0, 0, 0, 0, 0, 0);

        MockEngineBuilder::default().with_max_clients(16).run(|| {
            Python::with_gil(|py| {
                let result = pyshinqlx_set_powerups(py, -1, &powerups);
                assert!(result.is_err_and(|err| err.is_instance_of::<PyValueError>(py)));
            });
        });
    }

    #[rstest]
    #[cfg_attr(miri, ignore)]
    #[serial]
    fn set_powerups_for_client_id_too_large(_pyshinqlx_setup: ()) {
        let powerups = Powerups(0, 0, 0, 0, 0, 0);

        MockEngineBuilder::default().with_max_clients(16).run(|| {
            Python::with_gil(|py| {
                let result = pyshinqlx_set_powerups(py, 666, &powerups);
                assert!(result.is_err_and(|err| err.is_instance_of::<PyValueError>(py)));
            });
        });
    }

    #[rstest]
    #[cfg_attr(miri, ignore)]
    #[serial]
    fn set_powerups_for_existing_game_client(_pyshinqlx_setup: ()) {
        let game_entity_from_ctx = MockGameEntity::from_context();
        game_entity_from_ctx.expect().returning(|_| {
            let mut mock_game_entity = MockGameEntity::new();
            mock_game_entity.expect_get_game_client().returning(|| {
                let mut mock_game_client = MockGameClient::new();
                mock_game_client
                    .expect_set_powerups()
                    .with(predicate::eq([1, 2, 3, 4, 5, 6]))
                    .times(1);
                Ok(mock_game_client)
            });
            mock_game_entity
        });

        let powerups = Powerups(1, 2, 3, 4, 5, 6);

        MockEngineBuilder::default().with_max_clients(16).run(|| {
            let result = Python::with_gil(|py| pyshinqlx_set_powerups(py, 2, &powerups));
            assert_eq!(result.expect("result was not OK"), true);
        });
    }

    #[rstest]
    #[cfg_attr(miri, ignore)]
    #[serial]
    fn set_powerups_for_entity_with_no_game_client(_pyshinqlx_setup: ()) {
        let game_entity_from_ctx = MockGameEntity::from_context();
        game_entity_from_ctx.expect().returning(|_| {
            let mut mock_game_entity = MockGameEntity::new();
            mock_game_entity
                .expect_get_game_client()
                .returning(|| Err(QuakeLiveEngineError::MainEngineNotInitialized));
            mock_game_entity
        });

        let powerups = Powerups(0, 0, 0, 0, 0, 0);

        MockEngineBuilder::default().with_max_clients(16).run(|| {
            let result = Python::with_gil(|py| pyshinqlx_set_powerups(py, 2, &powerups));
            assert_eq!(result.expect("result was not OK"), false);
        });
    }
}

==> shinqlx-0.6.1.tar.gz: shinqlx-0.6.1/src/ffi/python/embed/set_powerups.rs <==
use super::validate_client_id;
use crate::ffi::{c::prelude::*, python::prelude::*};

/// Sets a player's powerups.
#[pyfunction]
#[pyo3(name = "set_powerups")]
pub(crate) fn pyshinqlx_set_powerups(
    py: Python<'_>,
    client_id: i32,
    powerups: &Powerups,
) -> PyResult<bool> {
    py.allow_threads(|| {
        validate_client_id(client_id)?;

        #[cfg_attr(
            test,
            allow(clippy::unnecessary_fallible_conversions, irrefutable_let_patterns)
        )]
        let opt_game_client = GameEntity::try_from(client_id)
            .ok()
            .and_then(|game_entity| game_entity.get_game_client().ok());
        let returned = opt_game_client.is_some();
        if let Some(mut game_client) = opt_game_client {
            game_client.set_powerups((*powerups).into());
        }
        Ok(returned)
    })
}

#[cfg(test)]
mod set_powerups_tests {
    use mockall::predicate;
    use pretty_assertions::assert_eq;
    use pyo3::exceptions::{PyEnvironmentError, PyValueError};
    use rstest::rstest;

    use crate::{
        ffi::{c::prelude::*, python::prelude::*},
        prelude::*,
    };

    #[rstest]
    #[cfg_attr(miri, ignore)]
    #[serial]
    fn set_powerups_when_main_engine_not_initialized(_pyshinqlx_setup: ()) {
        let powerups = Powerups(0, 0, 0, 0, 0, 0);

        Python::with_gil(|py| {
            let result = pyshinqlx_set_powerups(py, 21, &powerups);
            assert!(result.is_err_and(|err| err.is_instance_of::<PyEnvironmentError>(py)));
        });
    }

    #[rstest]
    #[cfg_attr(miri, ignore)]
    #[serial]
    fn set_powerups_for_client_id_too_small(_pyshinqlx_setup: ()) {
        let powerups = Powerups(0, 0, 0, 0, 0, 0);

        MockEngineBuilder::default().with_max_clients(16).run(|| {
            Python::with_gil(|py| {
                let result = pyshinqlx_set_powerups(py, -1, &powerups);
                assert!(result.is_err_and(|err| err.is_instance_of::<PyValueError>(py)));
            });
        });
    }

    #[rstest]
    #[cfg_attr(miri, ignore)]
    #[serial]
    fn set_powerups_for_client_id_too_large(_pyshinqlx_setup: ()) {
        let powerups = Powerups(0, 0, 0, 0, 0, 0);

        MockEngineBuilder::default().with_max_clients(16).run(|| {
            Python::with_gil(|py| {
                let result = pyshinqlx_set_powerups(py, 666, &powerups);
                assert!(result.is_err_and(|err| err.is_instance_of::<PyValueError>(py)));
            });
        });
    }

    #[rstest]
    #[cfg_attr(miri, ignore)]
    #[serial]
    fn set_powerups_for_existing_game_client(_pyshinqlx_setup: ()) {
        let powerups = Powerups(1, 2, 3, 4, 5, 6);

        MockGameEntityBuilder::default()
            .with_game_client(|| {
                let mut mock_game_client = MockGameClient::new();
                mock_game_client
                    .expect_set_powerups()
                    .with(predicate::eq([1, 2, 3, 4, 5, 6]))
                    .times(1);
                Ok(mock_game_client)
            })
            .run(predicate::always(), || {
                MockEngineBuilder::default().with_max_clients(16).run(|| {
                    let result = Python::with_gil(|py| pyshinqlx_set_powerups(py, 2, &powerups));
                    assert_eq!(result.expect("result was not OK"), true);
                });
            });
    }

    #[rstest]
    #[cfg_attr(miri, ignore)]
    #[serial]
    fn set_powerups_for_entity_with_no_game_client(_pyshinqlx_setup: ()) {
        let powerups = Powerups(0, 0, 0, 0, 0, 0);

        MockGameEntityBuilder::default()
            .with_game_client(|| Err(QuakeLiveEngineError::MainEngineNotInitialized))
            .run(predicate::always(), || {
                MockEngineBuilder::default().with_max_clients(16).run(|| {
                    let result = Python::with_gil(|py| pyshinqlx_set_powerups(py, 2, &powerups));
                    assert_eq!(result.expect("result was not OK"), false);
                });
            });
    }
}
